﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

using HidLibrary;
using MonoGameDemo.InputManagement;

namespace MonoGameDemo
{
    /// <summary>
    /// This is the main type for your game.
    /// </summary>
    public class Game1 : Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        HidDevice[] _deviceList;
        HidDevice _selectedDevice;
        SpriteFont font;
        MouseState oldstate;
        InputManager inputManager;

        List<Rectangle> Buttons = new List<Rectangle>();

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 960;
            graphics.GraphicsProfile = GraphicsProfile.HiDef;

            IsMouseVisible = true;
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            inputManager = new InputManager();

            _deviceList = HidDevices.Enumerate().ToArray();
            for (int i=0; i<_deviceList.Length; i++)
            {
                Rectangle r = new Rectangle(0, i * 27, 300, 25);
                Buttons.Add(r);
            }
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            font = Content.Load<SpriteFont>("Font");
            
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// game-specific content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
                Exit();

            MouseState ms = Mouse.GetState();
            if ((ms.LeftButton == ButtonState.Released) && (oldstate.LeftButton == ButtonState.Pressed))
            {
                for (int i =0; i<Buttons.Count; i++)
                {
                    if (Buttons[i].Contains(ms.X,ms.Y))
                    {
                        if (_selectedDevice != null)
                        {
                            _selectedDevice.CloseDevice();
                            
                        }
                        _selectedDevice = _deviceList[i];
                        InputManager.ConnectToDevice(_selectedDevice);
                        break;
                    }
                }

            }
            oldstate = ms;
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            MouseState ms = Mouse.GetState();

            spriteBatch.Begin();
            foreach (Rectangle r in Buttons)
            {
                if (r.Contains(ms.X,ms.Y))
                    spriteBatch.FillRectangle(r, Color.DarkGray);
                else
                    spriteBatch.FillRectangle(r, Color.Black);
            }
            spriteBatch.End();

            spriteBatch.Begin();
            Vector2 pos = new Vector2(10, 5);
            for (int i=0; i<_deviceList.Length; i++)
            {
                spriteBatch.DrawString(font, _deviceList[i].Description, pos, Color.White);
                pos.Y += 27;
            }
            spriteBatch.End();
            
            if (_selectedDevice != null)
            {
                spriteBatch.Begin();
                int x = 350;
                int y = 10;
                int maxy = 0;
                foreach (InputField ip in _selectedDevice.Capabilities.Fields)
                {
                    Vector2 size = font.MeasureString(ip.Name);
                    Vector2 dpos = new Vector2();
                    switch (ip.Length)
                    {
                        case 1: // Button
                            {

                                dpos.X = (x + 8) - (size.X * 0.5f);
                                dpos.Y = y + 8 + (size.Y);

                                if (dpos.X + size.X > GraphicsDevice.Viewport.Width)
                                {
                                    x = 350;
                                    y += maxy;
                                    y += 20;

                                    dpos.X = (x + 8) - (size.X * 0.5f);
                                    dpos.Y = y + 8 + (size.Y * 0.5f);
                                }

                                
                                Rectangle r = new Rectangle(x, y, 16, 16);
                                if (InputManager.GetNamedBool(_selectedDevice.Description,ip.Name))
                                    spriteBatch.FillRectangle(r, Color.Green);
                                else
                                    spriteBatch.FillRectangle(r, Color.Red);

                                
                                spriteBatch.DrawString(font, ip.Name, dpos, Color.White);
                                x += (int)(size.X + 8);
                                if (size.Y + 8 > maxy)
                                    maxy = 8 + (int)size.Y;
                               
                            }
                            break;

                        case 4:
                            {
                                if (ip.Name.Contains("Hat"))
                                {
                                    HiHat h = InputManager.GetNamedHiHat(_selectedDevice.Description, ip.Name);
                                    dpos.X = (x + 32) - (size.X * 0.5f);
                                    dpos.Y = y + 8 + (size.Y);

                                    if (dpos.X + size.X > GraphicsDevice.Viewport.Width)
                                    {
                                        x = 350;
                                        y += maxy;
                                        y += 20;

                                        dpos.X = (x + 32) - (size.X * 0.5f);
                                        dpos.Y = y + 8 + (size.Y * 0.5f);
                                    }

                                    spriteBatch.DrawCircle(new Vector2(x + 16 , y + 16), 16, 32, Color.White);
                                    int xp = x + 16;
                                    if (h.Left)
                                        xp -= 16;
                                    if (h.Right)
                                        xp += 16;
                                    int yp = y + 16;
                                    if (h.Up)
                                        yp -= 16;
                                    if (h.Down)
                                        yp += 16;
                                    spriteBatch.DrawCircle(new Vector2(xp, yp), 8, 8, Color.Red);

                                    spriteBatch.DrawString(font, ip.Name, dpos, Color.White);
                                    x += (int)(size.X + 8);
                                    if (size.Y + 8 > maxy)
                                        maxy = 8 + (int)size.Y;

                                }
                                else
                                {
                                    dpos.X = (x + 32) - (size.X * 0.5f);
                                    dpos.Y = y + 8 + (size.Y);

                                    if (dpos.X + size.X > GraphicsDevice.Viewport.Width)
                                    {
                                        x = 350;
                                        y += maxy;
                                        y += 20;

                                        dpos.X = (x + 32) - (size.X * 0.5f);
                                        dpos.Y = y + 8 + (size.Y * 0.5f);
                                    }

                                    int value = InputManager.GetNamedInt(_selectedDevice.Description, ip.Name);
                                    Rectangle r = new Rectangle(x, y, 64, 16);
                                    spriteBatch.FillRectangle(r, Color.Black);
                                    r.Width = value / 4;
                                    spriteBatch.FillRectangle(r, Color.Green);

                                    spriteBatch.DrawString(font, ip.Name, dpos, Color.White);
                                    x += (int)(size.X + 8);
                                    if (size.Y + 8 > maxy)
                                        maxy = 8 + (int)size.Y;
                                }
                            }
                            break;

                        case 8:
                            {
                                dpos.X = (x + 32) - (size.X * 0.5f);
                                dpos.Y = y + 8 + (size.Y);

                                if (dpos.X + size.X > GraphicsDevice.Viewport.Width)
                                {
                                    x = 350;
                                    y += maxy;
                                    y += 20;

                                    dpos.X = (x + 32) - (size.X * 0.5f);
                                    dpos.Y = y + 8 + (size.Y * 0.5f);
                                }

                                int value = InputManager.GetNamedInt(_selectedDevice.Description, ip.Name);
                                Rectangle r = new Rectangle(x, y, 64, 16);
                                spriteBatch.FillRectangle(r, Color.Black);
                                r.Width = value / 4;
                                spriteBatch.FillRectangle(r, Color.Green);

                                spriteBatch.DrawString(font, ip.Name, dpos, Color.White);
                                x += (int)(size.X + 8);
                                if (size.Y + 8 > maxy)
                                    maxy = 8 + (int)size.Y;
                            }
                            break;

                        default:
                            {
                                dpos.X = (x + 32) - (size.X * 0.5f);
                                dpos.Y = y + 8 + (size.Y);

                                if (dpos.X + size.X > GraphicsDevice.Viewport.Width)
                                {
                                    x = 350;
                                    y += maxy;
                                    y += 20;

                                    dpos.X = (x + 32) - (size.X * 0.5f);
                                    dpos.Y = y + 8 + (size.Y * 0.5f);
                                }

                                float value = InputManager.GetNamedFloat(_selectedDevice.Description, ip.Name);
                                Rectangle r = new Rectangle(x, y, 64, 16);
                                spriteBatch.FillRectangle(r, Color.Black);
                                r.Width = (int)(value * 64);
                                spriteBatch.FillRectangle(r, Color.Green);

                                spriteBatch.DrawString(font, ip.Name, dpos, Color.White);
                                x += (int)(size.X + 8);
                                if (size.Y + 8 > maxy)
                                    maxy = 8 + (int)size.Y;
                            }
                            break;
                    }
                    
                }
                spriteBatch.End();
            }

            base.Draw(gameTime);
        }
    }
}
