﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;

using HidLibrary;
using MonoGameDemo.InputManagement.BitStream;


namespace MonoGameDemo.InputManagement
{
    public class HIDDeviceHandler
    {
        HidDevice Device;
        HidFastReadDevice fastReader;
        Thread updateThread;
        volatile bool scan = true;

        public HIDDeviceHandler(HidDevice device)
        {
            Device = device;

            foreach (InputField i in Device.Capabilities.Fields)
            {
                switch (i.Length)
                {
                    case 1:
                        InputManager.RegisterBool(device.Description, i.Name);
                        break;
                    case 8:
                        InputManager.RegisterInt(device.Description, i.Name);
                        break;
                    case 4:
                        if (i.Name.Contains("Hat"))
                        {
                            InputManager.RegisterHiHat(device.Description, i.Name);
                        }
                        else
                        {
                            InputManager.RegisterInt(device.Description, i.Name);
                        }
                        break;
                    default:
                        InputManager.RegisterFloat(device.Description, i.Name);
                        break;
                }
            }


            HidFastReadEnumerator enumerator = new HidFastReadEnumerator();
            fastReader = (HidFastReadDevice)enumerator.GetDevice(device.DevicePath);
            fastReader.OpenDevice();
            fastReader.MonitorDeviceEvents = false;
            fastReader.Removed += Device_Removed;
            fastReader.FastReadReport(ReadProcess);

            updateThread = new Thread(() => timer_elapsed(this));
            updateThread.IsBackground = true;
            updateThread.Start();
        }

        public void CloseDown()
        {
            scan = false;
        }
        private void Device_Removed()
        {
            scan = false;
        }

        private static void timer_elapsed(HIDDeviceHandler parent)
        {
            bool com = parent.scan;
            while (com)
            {

                com = parent.scan;
                if (parent.fastReader != null)
                {
                    HidDeviceData dat = parent.fastReader.FastRead();
                    if (dat.Status == HidDeviceData.ReadStatus.Success)
                    {
                        HidReport report = new HidReport(parent.fastReader.Capabilities.InputReportByteLength, dat);
                        parent.ReadProcess(report);

                    }
                }
                else
                {
                    com = false;
                }
                Thread.Sleep(8);
               
            }
            parent.fastReader.Dispose();
        }

        public void ReadProcess(HidReport report)
        {
            BitStream.BitStream bs = new BitStream.BitStream(report.Data);
            foreach(InputField ip in Device.Capabilities.Fields)
            {
                switch (ip.Length)
                {
                    case 1:
                        {
                            bs.Seek(0, ip.StartBit);
                            bool res = bs.ReadBit().AsBool();
                            InputManager.SetNamedBool(Device.Description, ip.Name, res);
                        }
                        break;
                    case 8:
                        {
                            bs.Seek(0, ip.StartBit);
                            int res = bs.ReadByte();
                            InputManager.SetNamedInt(Device.Description, ip.Name, res);
                        }
                        break;
                    case 4:
                        {
                            bs.Seek(0, ip.StartBit);
                            Bit[] bits = bs.ReadBits(4);
                            int res = (bits[0].AsInt()) + (bits[1].AsInt() * 2) + (bits[2].AsInt() * 4) + (bits[3].AsInt() * 8);
                            if (ip.Name.Contains("Hat"))
                            {
                                InputManager.SetNamedHiHat(Device.Description, ip.Name, res);
                            }
                            else
                            {
                                InputManager.SetNamedInt(Device.Description, ip.Name, res);
                            }
                        }
                        break;

                    default:
                        {
                            bs.Seek(0, ip.StartBit);
                            Bit[] bits = bs.ReadBits(ip.Length);
                            float res = 0;
                            float bit = 1;
                            for (int i=0; i<ip.Length; i++)
                            {
                                res += bits[i].AsInt() * bit;
                                bit *= 2;
                            }
                            res /= (float)Math.Pow(2, ip.Length);       // convert to 0-1;
                            InputManager.SetNamedFloat(Device.Description, ip.Name, res);
                        }
                        break;
                }
            }
            bs = null;
        }
    }
}
